@@ -4,7 +4,7 @@
 #       Massachusetts Institute of Technology.
 # written by Olivier Thereaux <ot@w3.org> for W3C
 #
-# $Id: Basic.pm,v 1.13 2005/09/09 06:33:11 ot Exp $
+# $Id: Basic.pm,v 1.15 2006/06/23 03:53:38 ot Exp $
 
 package W3C::LogValidator::Basic;
 use strict;
@@ -16,7 +16,7 @@ our @ISA = qw(Exporter);
 our %EXPORT_TAGS = ( 'all' => [ qw() ] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
-our $VERSION = sprintf "%d.%03d",q$Revision: 1.13 $ =~ /(\d+)\.(\d+)/;
+our $VERSION = sprintf "%d.%03d",q$Revision: 1.15 $ =~ /(\d+)\.(\d+)/;
 
 
 ###########################
@@ -101,15 +101,17 @@ sub process_list
 
 	print "Now Using the Basic module... \n" if $verbose;
 	my %hits;
+	my %HTTPcodes;
 	my @uris = undef;
+	use DB_File; 
 	if (defined ($config{tmpfile}))
 	{
-		use DB_File; 
 		my $tmp_file = $config{tmpfile};
 		tie (%hits, 'DB_File', "$tmp_file", O_RDONLY) || 
 		    die ("Cannot create or open $tmp_file");
 		@uris = sort { $hits{$b} <=> $hits{$a} } keys %hits;
 	}
+
 	elsif ($self->uris())
 	{
 		@uris = $self->uris();
@@ -118,6 +120,15 @@ sub process_list
 
         @uris = $self->trim_uris(@uris);
 
+
+	if (defined ($config{tmpfile_HTTP_codes}))
+	{
+		my $tmp_file_HTTP_codes = $config{tmpfile_HTTP_codes};
+		tie (%HTTPcodes, 'DB_File', "$tmp_file_HTTP_codes", O_RDONLY) || 
+		    die ("Cannot create or open $tmp_file_HTTP_codes");
+	}
+
+
 	my $intro="Here are the <census> most popular documents overall for $name.";
 	my @result;
 	my @result_head;
@@ -130,11 +141,27 @@ sub process_list
 		my $uri = shift (@uris);
 		chomp ($uri);
 		my @result_tmp;
-		$census++;
-		push @result_tmp, "$census";
-		push @result_tmp, "$hits{$uri}";
-		push @result_tmp, "$uri";
-		push @result, [@result_tmp];
+		if (!defined $HTTPcodes{$uri})
+#(!defined $HTTPcodes{$uri}) or ($HTTPcodes{$uri} eq "") or ( $HTTPcodes{$uri} =~ /^[2-3]/))
+		{ # This module should ignore requests that resulted in 4XX and 5XX codes
+		    $census++;
+		    push @result_tmp, "$census";
+		    push @result_tmp, "$hits{$uri}";
+		    push @result_tmp, "$uri";
+		    push @result, [@result_tmp];
+		}
+		elsif ($HTTPcodes{$uri} eq "200") 
+		# should perhaps make a subroutine for that instead of DUPing code
+		{
+		    $census++;
+		    push @result_tmp, "$census";
+		    push @result_tmp, "$hits{$uri}";
+		    push @result_tmp, "$uri";
+		    push @result, [@result_tmp];
+		}
+		elsif ((defined $HTTPcodes{$uri}) and ($verbose > 1)) { 
+		    print "$uri returned code $HTTPcodes{$uri}, ignoring \n"; 
+		}
 	}
 	print "Done!\n" if $verbose;
 	if ($census eq 1) # let's repect grammar here
